package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.commom.utils.AccountValidatorUtil;
import com.community.commom.utils.BeanTools;

import com.community.huiju.dao.TaUserMapper;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.TaUser;

import com.community.huiju.dao.TpBuildingOwnerInfoMapper;
import com.community.huiju.model.TpBuildingOwnerInfo;

import com.community.huiju.service.IBuildingOwnerInfoService;
import com.google.common.collect.Maps;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 楼栋业主资料信息表 服务实现类
 * </p>
 *
 * @author weiximei
 * @since 2018-12-18
 */
@Service("iBuildingOwnerInfoService")
@Slf4j
public class BuildingOwnerInfoServiceImpl extends ServiceImpl<TpBuildingOwnerInfoMapper, TpBuildingOwnerInfo> implements IBuildingOwnerInfoService {

    @Autowired
    private TpBuildingOwnerInfoMapper tpBuildingOwnerInfoMapper;
    
    @Autowired
    private TaUserMapper taUserMapper;
    
    public static final Logger logger = LoggerFactory.getLogger(BuildingOwnerInfoServiceImpl.class);
    
    @Override
    public ResponseBean listQuery(String parameter) {

        ResponseBean responseBean = new ResponseBean();

        JSONObject jsonObject = JSONObject.parseObject(parameter);
        TpBuildingOwnerInfo tpBuildingOwnerInfo = jsonObject.toJavaObject(TpBuildingOwnerInfo.class);

        Long pageNum = jsonObject.getLong("pageNum") == null ? 1 : jsonObject.getLong("pageNum");
        Long pageSize = jsonObject.getLong("pageSize") == null ? 10 : jsonObject.getLong("pageSize");

        Page<TpBuildingOwnerInfo> page = new Page(pageNum,pageSize);

        Map<String, Object> map = Maps.newHashMap();
        // 栋
        map.put("building", tpBuildingOwnerInfo.getBuilding());
        // 单元
        map.put("unit", tpBuildingOwnerInfo.getUnit());
        // 楼层
        map.put("level", tpBuildingOwnerInfo.getLevel());
        // 房号
        map.put("room_no", tpBuildingOwnerInfo.getRoomNo());

        QueryWrapper<TpBuildingOwnerInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.allEq(map,false);
        queryWrapper.like(tpBuildingOwnerInfo.getOwnerName() != null,"owner_name", tpBuildingOwnerInfo.getOwnerName());

        // 分页查询
        IPage<TpBuildingOwnerInfo> infoIPage = tpBuildingOwnerInfoMapper.selectPage(page, queryWrapper);


        Map<String, Object> retMap = Maps.newHashMap();
        // 获取数据
        List<TpBuildingOwnerInfo> list = infoIPage.getRecords();

        retMap.put("data", list);
        retMap.put("total", infoIPage.getTotal());
        retMap.put("pageNum", infoIPage.getCurrent());
        retMap.put("pageSize", infoIPage.getSize());

        responseBean.addSuccess(list);

        return responseBean;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean update(String parameter, Integer userId) {
        ResponseBean responseBean = new ResponseBean();

        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(parameter, TpBuildingOwnerInfo.class);
        TpBuildingOwnerInfo oldTpBuildingOwnerInfo = tpBuildingOwnerInfoMapper.selectById(tpBuildingOwnerInfo.getId());
        if (null == oldTpBuildingOwnerInfo) {
            throw new WisdomException("%s 数据不存在!", tpBuildingOwnerInfo.getId());
        }

        BeanTools.copyProperties(tpBuildingOwnerInfo, oldTpBuildingOwnerInfo);

        oldTpBuildingOwnerInfo.setUpdateDate(LocalDateTime.now());
        oldTpBuildingOwnerInfo.setUpdateUser(userId);

        int row = tpBuildingOwnerInfoMapper.updateById(oldTpBuildingOwnerInfo);
        if (row <= 0) {
            throw new WisdomException("操作失败!");
        } else {
            responseBean.addSuccess("操作成功!");
        }
        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean add(String parameter, Integer userId) {
        ResponseBean responseBean = new ResponseBean();

        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(parameter, TpBuildingOwnerInfo.class);
        // 主键是自增的, 所以设置为 null
        tpBuildingOwnerInfo.setId(null);
        tpBuildingOwnerInfo.setUpdateUser(userId);
        tpBuildingOwnerInfo.setUpdateDate(LocalDateTime.now());
        tpBuildingOwnerInfo.setCreateDate(LocalDateTime.now());
        tpBuildingOwnerInfo.setCreateUser(userId);

        int row = tpBuildingOwnerInfoMapper.insert(tpBuildingOwnerInfo);
        if (row <= 0) {
            throw new WisdomException("操作失败!");
        } else {
            responseBean.addSuccess("操作成功!");
        }

        return responseBean;
    }

    @Override
    public ResponseBean getBuildingOrUnitOrNumber(String parameter,Integer communityId) {

        ResponseBean responseBean = new ResponseBean();
        if (null == communityId || "".equals(communityId)) {
            throw new WisdomException("编号:%s 小区不存在!", communityId);
        }

        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(parameter, TpBuildingOwnerInfo.class);

        Map<String, Object> map = Maps.newHashMap();

        // 小区
        map.put("community_id", communityId);
        // 栋
        map.put("building", tpBuildingOwnerInfo.getBuilding());
        // 单元
        map.put("unit", tpBuildingOwnerInfo.getUnit());
        // 楼层
        map.put("level", tpBuildingOwnerInfo.getLevel());

        QueryWrapper<TpBuildingOwnerInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.allEq(map, false);

        /**
         * 1.如果 楼栋值不存在表示根据小区Id查询时查询楼栋!
         * 2.如果 楼栋存在表示查询单元
         * 3.如果 单元存在表示查询楼层
         * 4.如果 楼层存在表示查询户号
         *
         * 查询户号, 依赖于前面的查询条件成立!
         *
         * 这里的查询都依赖于, 前面的查询条件成立! 比如 要使用条件2, 那么条件1 必须成立!
         *
         */

        String column = null;
        if (null == tpBuildingOwnerInfo.getBuilding()) {
            column = "building";
        } else if (null == tpBuildingOwnerInfo.getUnit()) {
            column = "unit";
        } else if (null == tpBuildingOwnerInfo.getLevel()){
            column = "level";
        } else if (null == tpBuildingOwnerInfo.getRoomNo()) {
            column = "room_no";
        }
        queryWrapper.groupBy(column);
        List<TpBuildingOwnerInfo> selectList = tpBuildingOwnerInfoMapper.selectList(queryWrapper);

        responseBean.addSuccess(selectList);

        return responseBean;
    }
    
    /**
     * 上传excel里面的内容
     *
     * @param file
     * @param userElement
     * @return
     */
    @Override
    public ResponseBean submitExcelData(MultipartFile file, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = getExcelData(file);
        if (!responseBean.getCode().equals(Constant.RESPONSE_SUCCESS)){
            return responseBean;
        }
        //获取要上传的数据
        Map<String,Object> map = (Map<String, Object>) responseBean.getData();
        List<TpBuildingOwnerInfo> list = (List<TpBuildingOwnerInfo>) map.get("list");
        list.stream().forEach(BuildingOwnerInfo -> {
            BuildingOwnerInfo.setCommunityId(userElement.getCommunityId());
            BuildingOwnerInfo.setVerifyStatus(Constant.UN_VERIFY_STATUS);
            BuildingOwnerInfo.setCreateUser(userElement.getId());
            BuildingOwnerInfo.setCreateDate(LocalDateTime.now());
            BuildingOwnerInfo.setUpdateUser(userElement.getId());
            BuildingOwnerInfo.setUpdateDate(LocalDateTime.now());
        });
        //批量插入
	    tpBuildingOwnerInfoMapper.batchInsert(list);
        //判断是否认证过
        List<TpBuildingOwnerInfo> updateList = new ArrayList<TpBuildingOwnerInfo>();
        List<TaUser> updataTaUserList = new ArrayList<TaUser>();
        list.stream().forEach(BuildingOwnerInfo -> {
            TaUser taUser = taUserMapper.selectByTel(BuildingOwnerInfo.getOwnerTel(),BuildingOwnerInfo.getCommunityId());
            if (null != taUser){
                BuildingOwnerInfo.setVerifyStatus(Constant.VERIFY_STATUS);
                updateList.add(BuildingOwnerInfo);
                taUser.setVerifyStatus(Constant.VERIFY_STATUS);
                taUser.setBuildingOwnerInfoId(BuildingOwnerInfo.getId());
                updataTaUserList.add(taUser);
            }
        });
        //批量更新
        if (updateList.size() > 0){
	        tpBuildingOwnerInfoMapper.batchUpdate(updateList);
        }
        if (updataTaUserList.size() > 0){
            taUserMapper.batchUpdate(updataTaUserList);
        }
        responseBean.addSuccess("success");
        return responseBean;
    }
    
    /**
     * 根据excel获取相关信息
     *
     * @param file
     * @return
     */
    @Override
    public ResponseBean getExcelData(MultipartFile file) {
        ResponseBean responseBean = new ResponseBean();
        List<TpBuildingOwnerInfo> list = new ArrayList<>();
        if (file == null) {
            responseBean.addError("对象不能为空");
            return responseBean;
        }
        //获取文件的名字
        String originalFilename = file.getOriginalFilename();
        Workbook workbook = null;
        try {
            if (originalFilename.endsWith(Constant.SUFFIX_2003)) {
                workbook = new HSSFWorkbook(file.getInputStream());
            } else if (originalFilename.endsWith(Constant.SUFFIX_2007)) {
                workbook = new XSSFWorkbook(file.getInputStream());
            }
        } catch (Exception e) {
            logger.info(originalFilename);
            e.printStackTrace();
            responseBean.addError("格式错误");
            return responseBean;
        }
        if (workbook == null) {
            logger.info(originalFilename);
            responseBean.addError("格式错误");
            return responseBean;
        } else {
            List<String> temBuildingList = new ArrayList<String>();
            List<String> temTelList = new ArrayList<String>();
            //获取一个sheet也就是一个工作簿
            Sheet sheet = workbook.getSheetAt(0);
            int lastRowNum = sheet.getLastRowNum();
            //从第一行开始第一行一般是标题
            for (int j = 3; j <= lastRowNum; j++) {
                Row row = sheet.getRow(j);
                TpBuildingOwnerInfo tpBuildingOwnerInfo = new TpBuildingOwnerInfo();
                String building = row.getCell(0).getStringCellValue();
                String unit = row.getCell(1).getStringCellValue();
                String level = row.getCell(2).getStringCellValue();
                String roomNo = row.getCell(3).getStringCellValue();
                String ownerName = row.getCell(4).getStringCellValue();
                String ownerTel = row.getCell(5).getStringCellValue();
                //校验手机号码
                if (!AccountValidatorUtil.isPhone(ownerTel)){
                    responseBean.addError("请输入正确手机号:" + ownerTel);
                    return responseBean;
                }
                String buildingInfo = building + unit + level + roomNo;
                if (temBuildingList.contains(buildingInfo)){
                    logger.info("存在重复房源:{}",buildingInfo);
                    responseBean.addError("存在重复房源:"+ buildingInfo);
                    return responseBean;
                }
                if (temTelList.contains(ownerTel)){
                    logger.info("存在重复手机号:{}",ownerTel);
                    responseBean.addError("存在重复手机号:"+ ownerTel);
                    return responseBean;
                }
                //把数据塞入temList校验是否有重复的数据
                temBuildingList.add(buildingInfo);
                temTelList.add(ownerTel);
                //构建数据
                tpBuildingOwnerInfo.setBuilding(building);
                tpBuildingOwnerInfo.setUnit(unit);
                tpBuildingOwnerInfo.setLevel(level);
                tpBuildingOwnerInfo.setRoomNo(roomNo);
                tpBuildingOwnerInfo.setOwnerName(ownerName);
                tpBuildingOwnerInfo.setOwnerTel(ownerTel);
                list.add(tpBuildingOwnerInfo);
            }
        }
        //构建分页
        Map<String,Object> data = new HashMap<String, Object>();
        data.put("list",list);
        data.put("total",list.size());
        
        responseBean.addSuccess(data);
        return responseBean;
    }

    @Override
    public ResponseBean selectUserApprove(String paramets) {

        ResponseBean responseBean = new ResponseBean();

        JSONObject jsonobject = JSONObject.parseObject(paramets);
        Integer pageNum = jsonobject.getInteger("pageNum");
        Integer pageSize = jsonobject.getInteger("pageSize");
        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(paramets, TpBuildingOwnerInfo.class);

        // 分页
        Page<TpBuildingOwnerInfo> page = new Page<>();
        page.setSize(pageSize == null ? 10 : pageSize);
        page.setCurrent(pageNum == null ? 1 : pageNum);

        List<TpBuildingOwnerInfo> mapList = tpBuildingOwnerInfoMapper.selectUserApprove(tpBuildingOwnerInfo.getOwnerName(),tpBuildingOwnerInfo.getOwnerTel(), page);
        responseBean.addSuccess(mapList);

        return responseBean;
    }
}
